@@ -11,6 +11,8 @@
 #   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
+import warnings
+
 from typing import (
     Callable,
     Dict,
@@ -31,8 +33,9 @@ import pandas as pd
 import scipy.sparse as sps
 
 from aeppl.logprob import CheckParameterValue
+from aeppl.transforms import RVTransform
 from aesara import scalar
-from aesara.compile.mode import Mode, get_mode
+from aesara.compile import Function, Mode, get_mode
 from aesara.gradient import grad
 from aesara.graph import node_rewriter, rewrite_graph
 from aesara.graph.basic import (
@@ -57,7 +60,7 @@ from aesara.tensor.random.var import (
 )
 from aesara.tensor.rewriting.basic import topo_constant_folding
 from aesara.tensor.rewriting.shape import ShapeFeature
-from aesara.tensor.sharedvar import SharedVariable
+from aesara.tensor.sharedvar import SharedVariable, TensorSharedVariable
 from aesara.tensor.subtensor import AdvancedIncSubtensor, AdvancedIncSubtensor1
 from aesara.tensor.var import TensorConstant, TensorVariable
 
@@ -205,10 +208,9 @@ def walk_model(
     yield from walk(graphs, expand, bfs=False)
 
 
-def replace_rvs_in_graphs(
+def _replace_rvs_in_graphs(
     graphs: Iterable[TensorVariable],
     replacement_fn: Callable[[TensorVariable], Dict[TensorVariable, TensorVariable]],
-    initial_replacements: Optional[Dict[TensorVariable, TensorVariable]] = None,
     **kwargs,
 ) -> Tuple[List[TensorVariable], Dict[TensorVariable, TensorVariable]]:
     """Replace random variables in graphs
@@ -226,8 +228,6 @@ def replace_rvs_in_graphs(
     that were made.
     """
     replacements = {}
-    if initial_replacements:
-        replacements.update(initial_replacements)
 
     def expand_replace(var):
         new_nodes = []
@@ -239,6 +239,7 @@ def replace_rvs_in_graphs(
             new_nodes.extend(replacement_fn(var, replacements))
         return new_nodes
 
+    # This iteration populates the replacements
     for var in walk_model(graphs, expand_fn=expand_replace, **kwargs):
         pass
 
@@ -253,7 +254,15 @@ def replace_rvs_in_graphs(
             clone=False,
         )
 
-        fg.replace_all(replacements.items(), import_missing=True)
+        # replacements have to be done in reverse topological order so that nested
+        # expressions get recursively replaced correctly
+        toposort = fg.toposort()
+        sorted_replacements = sorted(
+            tuple(replacements.items()),
+            key=lambda pair: toposort.index(pair[0].owner),
+            reverse=True,
+        )
+        fg.replace_all(sorted_replacements, import_missing=True)
 
         graphs = list(fg.outputs)
 
@@ -261,11 +270,10 @@ def replace_rvs_in_graphs(
 
 
 def rvs_to_value_vars(
-    graphs: Iterable[TensorVariable],
+    graphs: Iterable[Variable],
     apply_transforms: bool = True,
-    initial_replacements: Optional[Dict[TensorVariable, TensorVariable]] = None,
     **kwargs,
-) -> List[TensorVariable]:
+) -> List[Variable]:
     """Clone and replace random variables in graphs with their value variables.
 
     This will *not* recompute test values in the resulting graphs.
@@ -276,10 +284,11 @@ def rvs_to_value_vars(
         The graphs in which to perform the replacements.
     apply_transforms
         If ``True``, apply each value variable's transform.
-    initial_replacements
-        A ``dict`` containing the initial replacements to be made.
-
     """
+    warnings.warn(
+        "rvs_to_value_vars is deprecated. Use model.replace_rvs_by_values instead",
+        FutureWarning,
+    )
 
     def populate_replacements(
         random_var: TensorVariable, replacements: Dict[TensorVariable, TensorVariable]
@@ -311,15 +320,72 @@ def rvs_to_value_vars(
     equiv = clone_get_equiv(inputs, graphs, False, False, {})
     graphs = [equiv[n] for n in graphs]
 
-    if initial_replacements:
-        initial_replacements = {
-            equiv.get(k, k): equiv.get(v, v) for k, v in initial_replacements.items()
-        }
-
-    graphs, _ = replace_rvs_in_graphs(
+    graphs, _ = _replace_rvs_in_graphs(
         graphs,
         replacement_fn=populate_replacements,
-        initial_replacements=initial_replacements,
+        **kwargs,
+    )
+
+    return graphs
+
+
+def replace_rvs_by_values(
+    graphs: Sequence[TensorVariable],
+    *,
+    rvs_to_values: Dict[TensorVariable, TensorVariable],
+    rvs_to_transforms: Dict[TensorVariable, RVTransform],
+    **kwargs,
+) -> List[TensorVariable]:
+    """Clone and replace random variables in graphs with their value variables.
+
+    This will *not* recompute test values in the resulting graphs.
+
+    Parameters
+    ----------
+    graphs
+        The graphs in which to perform the replacements.
+    rvs_to_values
+        Mapping between the original graph RVs and respective value variables
+    rvs_to_transforms
+        Mapping between the original graph RVs and respective value transforms
+    """
+
+    # Clone original graphs so that we don't modify variables in place
+    inputs = [i for i in graph_inputs(graphs) if not isinstance(i, Constant)]
+    equiv = clone_get_equiv(inputs, graphs, False, False, {})
+    graphs = [equiv[n] for n in graphs]
+
+    # Get needed mappings for equivalent cloned variables
+    equiv_rvs_to_values = {}
+    equiv_rvs_to_transforms = {}
+    for rv, value in rvs_to_values.items():
+        equiv_rv = equiv.get(rv, rv)
+        equiv_rvs_to_values[equiv_rv] = equiv.get(value, value)
+        equiv_rvs_to_transforms[equiv_rv] = rvs_to_transforms[rv]
+
+    def poulate_replacements(rv, replacements):
+        # Populate replacements dict with {rv: value} pairs indicating which graph
+        # RVs should be replaced by what value variables.
+
+        # No value variable to replace RV with
+        value = equiv_rvs_to_values.get(rv, None)
+        if value is None:
+            return []
+
+        transform = equiv_rvs_to_transforms.get(rv, None)
+        if transform is not None:
+            # We want to replace uses of the RV by the back-transformation of its value
+            value = transform.backward(value, *rv.owner.inputs)
+            value.name = rv.name
+
+        replacements[rv] = value
+        # Also walk the graph of the value variable to make any additional
+        # replacements if that is not a simple input variable
+        return [value]
+
+    graphs, _ = _replace_rvs_in_graphs(
+        graphs,
+        replacement_fn=poulate_replacements,
         **kwargs,
     )
 
@@ -535,55 +601,155 @@ def make_shared_replacements(point, vars, model):
 
 def join_nonshared_inputs(
     point: Dict[str, np.ndarray],
-    xs: List[TensorVariable],
-    vars: List[TensorVariable],
-    shared,
-    make_shared: bool = False,
-):
+    outputs: List[TensorVariable],
+    inputs: List[TensorVariable],
+    shared_inputs: Optional[Dict[TensorVariable, TensorSharedVariable]] = None,
+    make_inputs_shared: bool = False,
+) -> Tuple[List[TensorVariable], TensorVariable]:
     """
-    Takes a list of Aesara Variables and joins their non shared inputs into a single input.
+    Create new outputs and input TensorVariables where the non-shared inputs are joined
+    in a single raveled vector input.
 
     Parameters
     ----------
-    point: a sample point
-    xs: list of Aesara tensors
-    vars: list of variables to join
+    point : dict of {str : array_like}
+        Dictionary that maps each input variable name to a numerical variable. The values
+        are used to extract the shape of each input variable to establish a correct
+        mapping between joined and original inputs. The shape of each variable is
+        assumed to be fixed.
+    outputs : list of TensorVariable
+        List of output TensorVariables whose non-shared inputs will be replaced
+        by a joined vector input.
+    inputs : list of TensorVariable
+        List of input TensorVariables which will be replaced by a joined vector input.
+    shared_inputs : dict of {TensorVariable : TensorSharedVariable}, optional
+        Dict of TensorVariable and their associated TensorSharedVariable in
+        subgraph replacement.
+    make_inputs_shared : bool, default False
+        Whether to make the joined vector input a shared variable.
 
     Returns
     -------
-    tensors, inarray
-    tensors: list of same tensors but with inarray as input
-    inarray: vector of inputs
+    new_outputs : list of TensorVariable
+        List of new outputs `outputs` TensorVariables that depend on `joined_inputs` and new shared variables as inputs.
+    joined_inputs : TensorVariable
+        Joined input vector TensorVariable for the `new_outputs`
+
+    Examples
+    --------
+    Join the inputs of a simple Aesara graph.
+
+    .. code-block:: python
+
+        import aesara.tensor as at
+        import numpy as np
+
+        from pymc.aesaraf import join_nonshared_inputs
+
+        # Original non-shared inputs
+        x = at.scalar("x")
+        y = at.vector("y")
+        # Original output
+        out = x + y
+        print(out.eval({x: np.array(1), y: np.array([1, 2, 3])})) # [2, 3, 4]
+
+        # New output and inputs
+        [new_out], joined_inputs = join_nonshared_inputs(
+            point={ # Only shapes matter
+                "x": np.zeros(()),
+                "y": np.zeros(3),
+            },
+            outputs=[out],
+            inputs=[x, y],
+        )
+        print(new_out.eval({
+            joined_inputs: np.array([1, 1, 2, 3]),
+        })) # [2, 3, 4]
+
+    Join the input value variables of a model logp.
+
+    .. code-block:: python
+
+        import pymc as pm
+
+        with pm.Model() as model:
+            mu_pop = pm.Normal("mu_pop")
+            sigma_pop = pm.HalfNormal("sigma_pop")
+            mu = pm.Normal("mu", mu_pop, sigma_pop, shape=(3, ))
+
+            y = pm.Normal("y", mu, 1.0, observed=[0, 1, 2])
+
+        print(model.compile_logp()({
+            "mu_pop": 0,
+            "sigma_pop_log__": 1,
+            "mu": [0, 1, 2],
+        })) # -12.691227342634292
+
+        initial_point = model.initial_point()
+        inputs = model.value_vars
+
+        [logp], joined_inputs = join_nonshared_inputs(
+            point=initial_point,
+            outputs=[model.logp()],
+            inputs=inputs,
+        )
+
+        print(logp.eval({
+            joined_inputs: [0, 1, 0, 1, 2],
+        })) # -12.691227342634292
+
+    Same as above but with the `mu_pop` value variable being shared.
+
+    .. code-block:: python
+
+        from aesara import shared
+
+        mu_pop_input, *other_inputs = inputs
+        shared_mu_pop_input = shared(0.0)
+
+        [logp], other_joined_inputs = join_nonshared_inputs(
+            point=initial_point,
+            outputs=[model.logp()],
+            inputs=other_inputs,
+            shared_inputs={
+                mu_pop_input: shared_mu_pop_input
+            },
+        )
+
+        print(logp.eval({
+            other_joined_inputs: [1, 0, 1, 2],
+        })) # -12.691227342634292
     """
-    if not vars:
-        raise ValueError("Empty list of variables.")
+    if not inputs:
+        raise ValueError("Empty list of input variables.")
 
-    joined = at.concatenate([var.ravel() for var in vars])
+    raveled_inputs = at.concatenate([var.ravel() for var in inputs])
 
-    if not make_shared:
-        tensor_type = joined.type
-        inarray = tensor_type("inarray")
+    if not make_inputs_shared:
+        tensor_type = raveled_inputs.type
+        joined_inputs = tensor_type("joined_inputs")
     else:
-        if point is None:
-            raise ValueError("A point is required when `make_shared` is True")
-        joined_values = np.concatenate([point[var.name].ravel() for var in vars])
-        inarray = aesara.shared(joined_values, "inarray")
+        joined_values = np.concatenate([point[var.name].ravel() for var in inputs])
+        joined_inputs = aesara.shared(joined_values, "joined_inputs")
 
     if aesara.config.compute_test_value != "off":
-        inarray.tag.test_value = joined.tag.test_value
+        joined_inputs.tag.test_value = raveled_inputs.tag.test_value
 
-    replace = {}
+    replace: Dict[TensorVariable, TensorVariable] = {}
     last_idx = 0
-    for var in vars:
+    for var in inputs:
         shape = point[var.name].shape
         arr_len = np.prod(shape, dtype=int)
-        replace[var] = inarray[last_idx : last_idx + arr_len].reshape(shape).astype(var.dtype)
+        replace[var] = joined_inputs[last_idx : last_idx + arr_len].reshape(shape).astype(var.dtype)
         last_idx += arr_len
 
-    replace.update(shared)
+    if shared_inputs is not None:
+        replace.update(shared_inputs)
 
-    xs_special = [aesara.clone_replace(x, replace, rebuild_strict=False) for x in xs]
-    return xs_special, inarray
+    new_outputs = [
+        aesara.clone_replace(output, replace, rebuild_strict=False) for output in outputs
+    ]
+    return new_outputs, joined_inputs
 
 
 class PointFunc:
@@ -878,7 +1044,7 @@ def compile_pymc(
     random_seed: SeedSequenceSeed = None,
     mode=None,
     **kwargs,
-) -> Callable[..., Union[np.ndarray, List[np.ndarray]]]:
+) -> Function:
     """Use ``aesara.function`` with specialized pymc rewrites always enabled.
 
     This function also ensures shared RandomState/Generator used by RandomVariables
