@@ -16,6 +16,7 @@ import warnings
 import aesara.tensor as at
 import numpy as np
 
+from aesara.tensor import TensorConstant
 from aesara.tensor.random.basic import (
     RandomVariable,
     ScipyRandomVariable,
@@ -139,20 +140,6 @@ class Binomial(Discrete):
         return mean
 
     def logp(value, n, p):
-        r"""
-        Calculate log-probability of Binomial distribution at specified value.
-
-        Parameters
-        ----------
-        value: numeric
-            Value(s) for which log-probability is calculated. If the log probabilities for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor
-
-        Returns
-        -------
-        TensorVariable
-        """
-
         res = at.switch(
             at.or_(at.lt(value, 0), at.gt(value, n)),
             -np.inf,
@@ -162,20 +149,6 @@ class Binomial(Discrete):
         return check_parameters(res, 0 <= n, 0 <= p, p <= 1, msg="n >= 0, 0 <= p <= 1")
 
     def logcdf(value, n, p):
-        """
-        Compute the log of the cumulative distribution function for Binomial distribution
-        at the specified value.
-
-        Parameters
-        ----------
-        value: numeric or np.ndarray or aesara.tensor
-            Value(s) for which log CDF is calculated. If the log CDF for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor.
-
-        Returns
-        -------
-        TensorVariable
-        """
         value = at.floor(value)
 
         res = at.switch(
@@ -270,19 +243,6 @@ class BetaBinomial(Discrete):
         return mean
 
     def logp(value, n, alpha, beta):
-        r"""
-        Calculate log-probability of BetaBinomial distribution at specified value.
-
-        Parameters
-        ----------
-        value: numeric
-            Value(s) for which log-probability is calculated. If the log probabilities for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor
-
-        Returns
-        -------
-        TensorVariable
-        """
         res = at.switch(
             at.or_(at.lt(value, 0), at.gt(value, n)),
             -np.inf,
@@ -291,19 +251,6 @@ class BetaBinomial(Discrete):
         return check_parameters(res, n >= 0, alpha > 0, beta > 0, msg="n >= 0, alpha > 0, beta > 0")
 
     def logcdf(value, n, alpha, beta):
-        """
-        Compute the log of the cumulative distribution function for BetaBinomial distribution
-        at the specified value.
-
-        Parameters
-        ----------
-        value: numeric
-            Value for which log CDF is calculated.
-
-        Returns
-        -------
-        TensorVariable
-        """
         # logcdf can only handle scalar values at the moment
         if np.ndim(value):
             raise TypeError(
@@ -395,20 +342,6 @@ class Bernoulli(Discrete):
         return at.switch(p < 0.5, 0, 1)
 
     def logp(value, p):
-        r"""
-        Calculate log-probability of Bernoulli distribution at specified value.
-
-        Parameters
-        ----------
-        value: numeric
-            Value(s) for which log-probability is calculated. If the log probabilities for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor
-
-        Returns
-        -------
-        TensorVariable
-        """
-
         res = at.switch(
             at.or_(at.lt(value, 0), at.gt(value, 1)),
             -np.inf,
@@ -418,20 +351,6 @@ class Bernoulli(Discrete):
         return check_parameters(res, p >= 0, p <= 1, msg="0 <= p <= 1")
 
     def logcdf(value, p):
-        """
-        Compute the log of the cumulative distribution function for Bernoulli distribution
-        at the specified value.
-
-        Parameters
-        ----------
-        value: numeric or np.ndarray or aesara.tensor
-            Value(s) for which log CDF is calculated. If the log CDF for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor.
-
-        Returns
-        -------
-        TensorVariable
-        """
         res = at.switch(
             at.lt(value, 0),
             -np.inf,
@@ -461,7 +380,7 @@ discrete_weibull = DiscreteWeibullRV()
 
 
 class DiscreteWeibull(Discrete):
-    R"""Discrete Weibull log-likelihood
+    R"""Discrete Weibull log-likelihood.
 
     The discrete Weibull distribution is a flexible model of count data that
     can handle both over- and under-dispersion.
@@ -470,6 +389,7 @@ class DiscreteWeibull(Discrete):
     .. math:: f(x \mid q, \beta) = q^{x^{\beta}} - q^{(x + 1)^{\beta}}
 
     .. plot::
+        :context: close-figs
 
         import matplotlib.pyplot as plt
         import numpy as np
@@ -498,6 +418,14 @@ class DiscreteWeibull(Discrete):
     Mean      :math:`\mu = \sum_{x = 1}^{\infty} q^{x^{\beta}}`
     Variance  :math:`2 \sum_{x = 1}^{\infty} x q^{x^{\beta}} - \mu - \mu^2`
     ========  ======================
+
+    Parameters
+    ----------
+    q : tensor_like of float
+        Shape parameter (0 < q < 1).
+    beta : tensor_like of float
+        Shape parameter (beta > 0).
+
     """
     rv_op = discrete_weibull
 
@@ -514,20 +442,6 @@ class DiscreteWeibull(Discrete):
         return median
 
     def logp(value, q, beta):
-        r"""
-        Calculate log-probability of DiscreteWeibull distribution at specified value.
-
-        Parameters
-        ----------
-        value: numeric
-            Value(s) for which log-probability is calculated. If the log probabilities for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor
-
-        Returns
-        -------
-        TensorVariable
-        """
-
         res = at.switch(
             at.lt(value, 0),
             -np.inf,
@@ -537,21 +451,6 @@ class DiscreteWeibull(Discrete):
         return check_parameters(res, 0 < q, q < 1, 0 < beta, msg="0 < q < 1, beta > 0")
 
     def logcdf(value, q, beta):
-        """
-        Compute the log of the cumulative distribution function for Discrete Weibull distribution
-        at the specified value.
-
-        Parameters
-        ----------
-        value: numeric or np.ndarray or aesara.tensor
-            Value(s) for which log CDF is calculated. If the log CDF for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor.
-
-        Returns
-        -------
-        TensorVariable
-        """
-
         res = at.switch(
             at.lt(value, 0),
             -np.inf,
@@ -619,19 +518,6 @@ class Poisson(Discrete):
         return mu
 
     def logp(value, mu):
-        r"""
-        Calculate log-probability of Poisson distribution at specified value.
-
-        Parameters
-        ----------
-        value: numeric
-            Value(s) for which log-probability is calculated. If the log probabilities for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor
-
-        Returns
-        -------
-        TensorVariable
-        """
         logprob = at.switch(
             at.lt(value, 0),
             -np.inf,
@@ -646,20 +532,6 @@ class Poisson(Discrete):
         return check_parameters(logprob, mu >= 0, msg="mu >= 0")
 
     def logcdf(value, mu):
-        """
-        Compute the log of the cumulative distribution function for Poisson distribution
-        at the specified value.
-
-        Parameters
-        ----------
-        value: numeric or np.ndarray or aesara.tensor
-            Value(s) for which log CDF is calculated. If the log CDF for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor.
-
-        Returns
-        -------
-        TensorVariable
-        """
         value = at.floor(value)
         # Avoid C-assertion when the gammaincc function is called with invalid values (#4340)
         safe_mu = at.switch(at.lt(mu, 0), 0, mu)
@@ -782,19 +654,6 @@ class NegativeBinomial(Discrete):
         return mu
 
     def logp(value, n, p):
-        r"""
-        Calculate log-probability of NegativeBinomial distribution at specified value.
-
-        Parameters
-        ----------
-        value: numeric
-            Value(s) for which log-probability is calculated. If the log probabilities for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor
-
-        Returns
-        -------
-        TensorVariable
-        """
         alpha = n
         mu = alpha * (1 - p) / p
 
@@ -819,20 +678,6 @@ class NegativeBinomial(Discrete):
         return at.switch(at.gt(alpha, 1e10), logp(Poisson.dist(mu=mu), value), negbinom)
 
     def logcdf(value, n, p):
-        """
-        Compute the log of the cumulative distribution function for NegativeBinomial distribution
-        at the specified value.
-
-        Parameters
-        ----------
-        value: numeric or np.ndarray or aesara.tensor
-            Value(s) for which log CDF is calculated. If the log CDF for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor.
-
-        Returns
-        -------
-        TensorVariable
-        """
         res = at.switch(
             at.lt(value, 0),
             -np.inf,
@@ -900,20 +745,6 @@ class Geometric(Discrete):
         return mean
 
     def logp(value, p):
-        r"""
-        Calculate log-probability of Geometric distribution at specified value.
-
-        Parameters
-        ----------
-        value: numeric
-            Value(s) for which log-probability is calculated. If the log probabilities for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor
-
-        Returns
-        -------
-        TensorVariable
-        """
-
         res = at.switch(
             at.lt(value, 1),
             -np.inf,
@@ -928,21 +759,6 @@ class Geometric(Discrete):
         )
 
     def logcdf(value, p):
-        """
-        Compute the log of the cumulative distribution function for Geometric distribution
-        at the specified value.
-
-        Parameters
-        ----------
-        value: numeric or np.ndarray or aesara.tensor
-            Value(s) for which log CDF is calculated. If the log CDF for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor.
-
-        Returns
-        -------
-        TensorVariable
-        """
-
         res = at.switch(
             at.lt(value, 0),
             -np.inf,
@@ -1021,24 +837,6 @@ class HyperGeometric(Discrete):
         return mode
 
     def logp(value, good, bad, n):
-        r"""
-        Calculate log-probability of HyperGeometric distribution at specified value.
-
-        Parameters
-        ----------
-        value : numeric
-            Value(s) for which log-probability is calculated. If the log probabilities for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor
-        good : integer, array_like or TensorVariable
-            Number of successful individuals in the population. Alias for parameter :math:`k`.
-        bad : integer, array_like or TensorVariable
-            Number of unsuccessful individuals in the population. Alias for :math:`N-k`.
-
-        Returns
-        -------
-        TensorVariable
-        """
-
         tot = good + bad
         result = (
             betaln(good + 1, 1)
@@ -1065,25 +863,6 @@ class HyperGeometric(Discrete):
         return check_parameters(res, lower <= upper, msg="lower <= upper")
 
     def logcdf(value, good, bad, n):
-        """
-        Compute the log of the cumulative distribution function for HyperGeometric distribution
-        at the specified value.
-
-        Parameters
-        ----------
-        value : numeric
-            Value for which log CDF is calculated.
-        good : integer
-            Number of successful individuals in the population. Alias for parameter :math:`k`.
-        bad : integer
-            Number of unsuccessful individuals in the population. Alias for :math:`N-k`.
-        n : integer
-            Number of samples drawn from the population (0 <= n <= N)
-
-        Returns
-        -------
-        TensorVariable
-        """
         # logcdf can only handle scalar values at the moment
         if np.ndim(value):
             raise TypeError(
@@ -1134,13 +913,14 @@ discrete_uniform = DiscreteUniformRV()
 
 
 class DiscreteUniform(Discrete):
-    R"""
-    Discrete uniform distribution.
+    R"""Discrete uniform distribution.
+
     The pmf of this distribution is
 
     .. math:: f(x \mid lower, upper) = \frac{1}{upper-lower+1}
 
     .. plot::
+        :context: close-figs
 
         import matplotlib.pyplot as plt
         import numpy as np
@@ -1167,9 +947,9 @@ class DiscreteUniform(Discrete):
 
     Parameters
     ----------
-    lower: int
+    lower : tensor_like of int
         Lower limit.
-    upper: int
+    upper : tensor_like of int
         Upper limit (upper > lower).
     """
 
@@ -1188,19 +968,6 @@ class DiscreteUniform(Discrete):
         return mode
 
     def logp(value, lower, upper):
-        r"""
-        Calculate log-probability of DiscreteUniform distribution at specified value.
-
-        Parameters
-        ----------
-        value: numeric
-            Value(s) for which log-probability is calculated. If the log probabilities for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor
-
-        Returns
-        -------
-        TensorVariable
-        """
         res = at.switch(
             at.or_(at.lt(value, lower), at.gt(value, upper)),
             -np.inf,
@@ -1209,21 +976,6 @@ class DiscreteUniform(Discrete):
         return check_parameters(res, lower <= upper, msg="lower <= upper")
 
     def logcdf(value, lower, upper):
-        """
-        Compute the log of the cumulative distribution function for Discrete uniform distribution
-        at the specified value.
-
-        Parameters
-        ----------
-        value: numeric or np.ndarray or aesara.tensor
-            Value(s) for which log CDF is calculated. If the log CDF for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor.
-
-        Returns
-        -------
-        TensorVariable
-        """
-
         res = at.switch(
             at.le(value, lower),
             -np.inf,
@@ -1246,6 +998,7 @@ class Categorical(Discrete):
     .. math:: f(x \mid p) = p_x
 
     .. plot::
+        :context: close-figs
 
         import matplotlib.pyplot as plt
         import numpy as np
@@ -1285,17 +1038,21 @@ class Categorical(Discrete):
         if logit_p is not None:
             p = pm.math.softmax(logit_p, axis=-1)
 
-        if isinstance(p, np.ndarray) or isinstance(p, list):
-            if (np.asarray(p) < 0).any():
-                raise ValueError(f"Negative `p` parameters are not valid, got: {p}")
-            p_sum = np.sum([p], axis=-1)
-            if not np.all(np.isclose(p_sum, 1.0)):
+        p = at.as_tensor_variable(p)
+        if isinstance(p, TensorConstant):
+            p_ = np.asarray(p.data)
+            if np.any(p_ < 0):
+                raise ValueError(f"Negative `p` parameters are not valid, got: {p_}")
+            p_sum_ = np.sum([p_], axis=-1)
+            if not np.all(np.isclose(p_sum_, 1.0)):
                 warnings.warn(
-                    f"`p` parameters sum to {p_sum}, instead of 1.0. They will be automatically rescaled. You can rescale them directly to get rid of this warning.",
+                    f"`p` parameters sum to {p_sum_}, instead of 1.0. "
+                    "They will be automatically rescaled. "
+                    "You can rescale them directly to get rid of this warning.",
                     UserWarning,
                 )
-                p = p / at.sum(p, axis=-1, keepdims=True)
-        p = at.as_tensor_variable(floatX(p))
+                p_ = p_ / at.sum(p_, axis=-1, keepdims=True)
+                p = at.as_tensor_variable(p_)
         return super().dist([p], **kwargs)
 
     def moment(rv, size, p):
@@ -1305,16 +1062,6 @@ class Categorical(Discrete):
         return mode
 
     def logp(value, p):
-        r"""
-        Calculate log-probability of Categorical distribution at specified value.
-
-        Parameters
-        ----------
-        value: numeric
-            Value(s) for which log-probability is calculated. If the log probabilities for multiple
-            values are desired the values must be provided in a numpy array or `TensorVariable`
-
-        """
         k = at.shape(p)[-1]
         p_ = p
         value_clip = at.clip(value, 0, k - 1)
@@ -1341,7 +1088,11 @@ class Categorical(Discrete):
         )
 
         return check_parameters(
-            res, at.all(p_ >= 0, axis=-1), at.all(p <= 1, axis=-1), msg="0 <= p <=1"
+            res,
+            p_ >= 0,
+            p_ <= 1,
+            at.isclose(at.sum(p, axis=-1), 1),
+            msg="0 <= p <=1, sum(p) = 1",
         )
 
 
@@ -1392,19 +1143,6 @@ class DiracDelta(Discrete):
         return c
 
     def logp(value, c):
-        r"""
-        Calculate log-probability of DiracDelta distribution at specified value.
-
-        Parameters
-        ----------
-        value: numeric
-            Value(s) for which log-probability is calculated. If the log probabilities for multiple
-            values are desired the values must be provided in a numpy array or Aesara tensor
-
-        Returns
-        -------
-        TensorVariable
-        """
         return at.switch(
             at.eq(value, c),
             at.zeros_like(value),
@@ -1531,6 +1269,7 @@ class ZeroInflatedBinomial:
             \end{array} \right.
 
     .. plot::
+        :context: close-figs
 
         import matplotlib.pyplot as plt
         import numpy as np
@@ -1606,6 +1345,7 @@ class ZeroInflatedNegativeBinomial:
        \right.
 
     .. plot::
+        :context: close-figs
 
         import matplotlib.pyplot as plt
         import numpy as np
