@@ -49,7 +49,7 @@ from pymc.distributions.shape_utils import (
     shape_from_dims,
 )
 from pymc.printing import str_for_dist
-from pymc.util import UNSET
+from pymc.util import UNSET, _add_future_warning_tag
 from pymc.vartypes import string_types
 
 __all__ = [
@@ -63,9 +63,9 @@ __all__ = [
 
 DIST_PARAMETER_TYPES: TypeAlias = Union[np.ndarray, int, float, TensorVariable]
 
-vectorized_ppc = contextvars.ContextVar(
+vectorized_ppc: contextvars.ContextVar[Optional[Callable]] = contextvars.ContextVar(
     "vectorized_ppc", default=None
-)  # type: contextvars.ContextVar[Optional[Callable]]
+)
 
 PLATFORM = sys.platform
 
@@ -371,6 +371,7 @@ class Distribution(metaclass=DistributionMeta):
         rv_out.logp = _make_nice_attr_error("rv.logp(x)", "pm.logp(rv, x)")
         rv_out.logcdf = _make_nice_attr_error("rv.logcdf(x)", "pm.logcdf(rv, x)")
         rv_out.random = _make_nice_attr_error("rv.random()", "pm.draw(rv)")
+        _add_future_warning_tag(rv_out)
         return rv_out
 
 
@@ -576,6 +577,14 @@ class DensityDist(Distribution):
 
     def __new__(cls, name, *args, **kwargs):
         kwargs.setdefault("class_name", name)
+        if isinstance(kwargs.get("observed", None), dict):
+            raise TypeError(
+                "Since ``v4.0.0`` the ``observed`` parameter should be of type"
+                " ``pd.Series``, ``np.array``, or ``pm.Data``."
+                " Previous versions allowed passing distribution parameters as"
+                " a dictionary in ``observed``, in the current version these "
+                "parameters are positional arguments."
+            )
         return super().__new__(cls, name, *args, **kwargs)
 
     @classmethod
