@@ -50,6 +50,7 @@ __all__ = [
 
 from pymc.aesaraf import PotentialShapeType
 from pymc.exceptions import ShapeError
+from pymc.util import _add_future_warning_tag
 
 
 def to_tuple(shape):
@@ -600,6 +601,7 @@ def change_dist_size(
         new_size = tuple(new_size)  # type: ignore
 
     new_dist = _change_dist_size(dist.owner.op, dist, new_size=new_size, expand=expand)
+    _add_future_warning_tag(new_dist)
 
     new_dist.name = dist.name
     for k, v in dist.tag.__dict__.items():
