@@ -30,7 +30,7 @@ from aesara.tensor import TensorVariable
 import pymc as pm
 
 from pymc.aesaraf import floatX
-from pymc.distributions import joint_logp, logcdf, logp
+from pymc.distributions import logcdf, logp
 from pymc.distributions.discrete import _OrderedLogistic, _OrderedProbit
 from pymc.tests.distributions.util import (
     BaseTestDistributionRandom,
@@ -69,7 +69,7 @@ def categorical_logpdf(value, p):
     if value >= 0 and value <= len(p):
         return floatX(np.log(np.moveaxis(p, -1, 0)[value]))
     else:
-        return -inf
+        return -np.inf
 
 
 def invlogit(x, eps=sys.float_info.epsilon):
@@ -469,6 +469,15 @@ class TestMatchesScipy:
             {"n": NatSmall, "p": Unit, "psi": Unit},
         )
 
+    @pytest.mark.parametrize("n", [2, 3, 4])
+    def test_categorical(self, n):
+        check_logp(
+            pm.Categorical,
+            Domain(range(n), dtype="int64", edges=(0, n)),
+            {"p": Simplex(n)},
+            lambda value, p: categorical_logpdf(value, p),
+        )
+
     @aesara.config.change_flags(compute_test_value="raise")
     def test_categorical_bounds(self):
         with pm.Model():
@@ -488,42 +497,40 @@ class TestMatchesScipy:
             # entries if there is a single or pair number of negative values
             # and the rest are zero
             np.array([-1, -1, 0, 0]),
+            at.as_tensor_variable([-1, -1, 0, 0]),
         ],
     )
     def test_categorical_negative_p(self, p):
-        with pytest.raises(ValueError, match=f"{p}"):
+        with pytest.raises(ValueError, match="Negative `p` parameters are not valid"):
             with pm.Model():
                 x = pm.Categorical("x", p=p)
 
-    def test_categorical_negative_p_symbolic(self):
-        with pytest.raises(ParameterValueError):
-            value = np.array([[1, 1, 1]])
-            invalid_dist = pm.Categorical.dist(p=at.as_tensor_variable([-1, 0.5, 0.5]))
-            pm.logp(invalid_dist, value).eval()
-
-    def test_categorical_p_not_normalized_symbolic(self):
-        with pytest.raises(ParameterValueError):
-            value = np.array([[1, 1, 1]])
-            invalid_dist = pm.Categorical.dist(p=at.as_tensor_variable([2, 2, 2]))
-            pm.logp(invalid_dist, value).eval()
-
-    @pytest.mark.parametrize("n", [2, 3, 4])
-    def test_categorical(self, n):
-        check_logp(
-            pm.Categorical,
-            Domain(range(n), dtype="int64", edges=(0, n)),
-            {"p": Simplex(n)},
-            lambda value, p: categorical_logpdf(value, p),
-        )
-
     def test_categorical_p_not_normalized(self):
         # test UserWarning is raised for p vals that sum to more than 1
         # and normaliation is triggered
-        with pytest.warns(UserWarning, match="[5]"):
+        with pytest.warns(UserWarning, match="They will be automatically rescaled"):
             with pm.Model() as m:
                 x = pm.Categorical("x", p=[1, 1, 1, 1, 1])
         assert np.isclose(m.x.owner.inputs[3].sum().eval(), 1.0)
 
+    def test_categorical_negative_p_symbolic(self):
+        value = np.array([[1, 1, 1]])
+
+        x = at.scalar("x")
+        invalid_dist = pm.Categorical.dist(p=[x, x, x])
+
+        with pytest.raises(ParameterValueError):
+            pm.logp(invalid_dist, value).eval({x: -1 / 3})
+
+    def test_categorical_p_not_normalized_symbolic(self):
+        value = np.array([[1, 1, 1]])
+
+        x = at.scalar("x")
+        invalid_dist = pm.Categorical.dist(p=(x, x, x))
+
+        with pytest.raises(ParameterValueError):
+            pm.logp(invalid_dist, value).eval({x: 0.5})
+
     @pytest.mark.parametrize("n", [2, 3, 4])
     def test_orderedlogistic(self, n):
         with warnings.catch_warnings():
@@ -567,8 +574,8 @@ def test_orderedlogistic_dimensions(shape):
             p=p,
             observed=obs,
         )
-    ologp = joint_logp(ol, np.ones_like(obs), sum=True).eval() * loge
-    clogp = joint_logp(c, np.ones_like(obs), sum=True).eval() * loge
+    ologp = pm.logp(ol, np.ones_like(obs)).sum().eval() * loge
+    clogp = pm.logp(c, np.ones_like(obs)).sum().eval() * loge
     expected = -np.prod((size,) + shape)
 
     assert c.owner.inputs[3].ndim == (len(shape) + 1)
