@@ -13,6 +13,7 @@
 #   limitations under the License.
 import pickle
 import threading
+import traceback
 import unittest
 import warnings
 
@@ -28,6 +29,7 @@ import pytest
 import scipy.sparse as sps
 import scipy.stats as st
 
+from aeppl.transforms import IntervalTransform
 from aesara.graph import graph_inputs
 from aesara.tensor import TensorVariable
 from aesara.tensor.random.op import RandomVariable
@@ -39,10 +41,13 @@ import pymc as pm
 from pymc import Deterministic, Potential
 from pymc.blocking import DictToArrayBijection, RaveledVars
 from pymc.distributions import Normal, transforms
+from pymc.distributions.logprob import _joint_logp
+from pymc.distributions.transforms import log
 from pymc.exceptions import ImputationWarning, ShapeError, ShapeWarning
 from pymc.model import Point, ValueGradFunction, modelcontext
 from pymc.tests.helpers import SeededTest
 from pymc.tests.models import simple_model
+from pymc.util import _FutureWarningValidatingScratchpad
 
 
 class NewModel(pm.Model):
@@ -351,7 +356,7 @@ class TestValueGradFunction(unittest.TestCase):
 
         assert m["x2_missing"].type == gf._extra_vars_shared["x2_missing"].type
 
-        pnt = m.initial_point(seed=None).copy()
+        pnt = m.initial_point(random_seed=None).copy()
         del pnt["x2_missing"]
 
         res = [gf(DictToArrayBijection.map(Point(pnt, model=m))) for i in range(5)]
@@ -402,7 +407,7 @@ def test_multiple_observed_rv():
     assert not model["x"] == model["mu"]
     assert model["x"] == model["x"]
     assert model["x"] in model.observed_RVs
-    assert not model["x"] in model.value_vars
+    assert model["x"] not in model.value_vars
 
 
 def test_tempered_logp_dlogp():
@@ -495,10 +500,22 @@ def test_model_value_vars():
 def test_model_var_maps():
     with pm.Model() as model:
         a = pm.Uniform("a")
-        x = pm.Normal("x", a)
+        x = pm.Normal("x", a, total_size=5)
 
-    assert model.rvs_to_values == {a: a.tag.value_var, x: x.tag.value_var}
-    assert model.values_to_rvs == {a.tag.value_var: a, x.tag.value_var: x}
+    assert set(model.rvs_to_values.keys()) == {a, x}
+    a_value = model.rvs_to_values[a]
+    x_value = model.rvs_to_values[x]
+    assert a_value.owner is None
+    assert x_value.owner is None
+    assert model.values_to_rvs == {a_value: a, x_value: x}
+
+    assert set(model.rvs_to_transforms.keys()) == {a, x}
+    assert isinstance(model.rvs_to_transforms[a], IntervalTransform)
+    assert model.rvs_to_transforms[x] is None
+
+    assert set(model.rvs_to_total_sizes.keys()) == {a, x}
+    assert model.rvs_to_total_sizes[a] is None
+    assert model.rvs_to_total_sizes[x] == 5
 
 
 def test_make_obs_var():
@@ -531,12 +548,12 @@ def test_make_obs_var():
 
     dense_output = fake_model.make_obs_var(fake_distribution, dense_input, None, None)
     assert dense_output == fake_distribution
-    assert isinstance(dense_output.tag.observations, TensorConstant)
+    assert isinstance(fake_model.rvs_to_values[dense_output], TensorConstant)
     del fake_model.named_vars[fake_distribution.name]
 
     sparse_output = fake_model.make_obs_var(fake_distribution, sparse_input, None, None)
     assert sparse_output == fake_distribution
-    assert sparse.basic._is_sparse_variable(sparse_output.tag.observations)
+    assert sparse.basic._is_sparse_variable(fake_model.rvs_to_values[sparse_output])
     del fake_model.named_vars[fake_distribution.name]
 
     # Here the RandomVariable is split into observed/imputed and a Deterministic is returned
@@ -557,29 +574,23 @@ def test_initial_point():
         a = pm.Uniform("a")
         x = pm.Normal("x", a)
 
-    with pytest.warns(FutureWarning):
-        initial_point = model.test_point
-
-    assert all(var.name in initial_point for var in model.value_vars)
-
     b_initval = np.array(0.3, dtype=aesara.config.floatX)
 
     with pytest.warns(FutureWarning), model:
         b = pm.Uniform("b", testval=b_initval)
 
-    b_value_var = model.rvs_to_values[b]
-    b_initval_trans = b_value_var.tag.transform.forward(b_initval, *b.owner.inputs).eval()
+    b_initval_trans = model.rvs_to_transforms[b].forward(b_initval, *b.owner.inputs).eval()
 
     y_initval = np.array(-2.4, dtype=aesara.config.floatX)
 
     with model:
         y = pm.Normal("y", initval=y_initval)
 
-    assert a in model.initial_values
-    assert x in model.initial_values
-    assert model.initial_values[b] == b_initval
+    assert a in model.rvs_to_initial_values
+    assert x in model.rvs_to_initial_values
+    assert model.rvs_to_initial_values[b] == b_initval
     assert model.initial_point(0)["b_interval__"] == b_initval_trans
-    assert model.initial_values[y] == y_initval
+    assert model.rvs_to_initial_values[y] == y_initval
 
 
 def test_point_logps():
@@ -635,12 +646,8 @@ class TestCheckStartVals(SeededTest):
             b = pm.Uniform("b", lower=2.0, upper=3.0)
 
         start = {
-            "a_interval__": model.rvs_to_values[a]
-            .tag.transform.forward(0.3, *a.owner.inputs)
-            .eval(),
-            "b_interval__": model.rvs_to_values[b]
-            .tag.transform.forward(2.1, *b.owner.inputs)
-            .eval(),
+            "a_interval__": model.rvs_to_transforms[a].forward(0.3, *a.owner.inputs).eval(),
+            "b_interval__": model.rvs_to_transforms[b].forward(2.1, *b.owner.inputs).eval(),
         }
         model.check_start_vals(start)
 
@@ -651,9 +658,7 @@ class TestCheckStartVals(SeededTest):
 
         start = {
             "a_interval__": np.nan,
-            "b_interval__": model.rvs_to_values[b]
-            .tag.transform.forward(2.1, *b.owner.inputs)
-            .eval(),
+            "b_interval__": model.rvs_to_transforms[b].forward(2.1, *b.owner.inputs).eval(),
         }
         with pytest.raises(pm.exceptions.SamplingError):
             model.check_start_vals(start)
@@ -664,12 +669,8 @@ class TestCheckStartVals(SeededTest):
             b = pm.Uniform("b", lower=2.0, upper=3.0)
 
         start = {
-            "a_interval__": model.rvs_to_values[a]
-            .tag.transform.forward(0.3, *a.owner.inputs)
-            .eval(),
-            "b_interval__": model.rvs_to_values[b]
-            .tag.transform.forward(2.1, *b.owner.inputs)
-            .eval(),
+            "a_interval__": model.rvs_to_transforms[a].forward(0.3, *a.owner.inputs).eval(),
+            "b_interval__": model.rvs_to_transforms[b].forward(2.1, *b.owner.inputs).eval(),
             "c": 1.0,
         }
         with pytest.raises(KeyError):
@@ -725,7 +726,7 @@ def test_nested_model_coords():
         e = pm.Normal("e", a[None] + d[:, None], dims=("dim2", "dim1"))
     assert m1.coords is m2.coords
     assert m1.dim_lengths is m2.dim_lengths
-    assert set(m2.RV_dims) < set(m1.RV_dims)
+    assert set(m2.named_vars_to_dims) < set(m1.named_vars_to_dims)
 
 
 def test_shapeerror_from_set_data_dimensionality():
@@ -1139,225 +1140,290 @@ def missing_data(request):
         return pd.DataFrame([1, 2, np.nan, 4, np.nan])
 
 
-def test_missing(missing_data):
+class TestImputationMissingData:
+    "Test for Missing Data imputation"
 
-    with pm.Model() as model:
-        x = pm.Normal("x", 1, 1)
-        with pytest.warns(ImputationWarning):
-            _ = pm.Normal("y", x, 1, observed=missing_data)
-
-    assert "y_missing" in model.named_vars
-
-    test_point = model.initial_point()
-    assert not np.isnan(model.compile_logp()(test_point))
-
-    with model:
-        prior_trace = pm.sample_prior_predictive(return_inferencedata=False)
-    assert {"x", "y"} <= set(prior_trace.keys())
-
-
-def test_missing_with_predictors():
-    predictors = np.array([0.5, 1, 0.5, 2, 0.3])
-    data = np.ma.masked_values([1, 2, -1, 4, -1], value=-1)
-    with pm.Model() as model:
-        x = pm.Normal("x", 1, 1)
-        with pytest.warns(ImputationWarning):
-            y = pm.Normal("y", x * predictors, 1, observed=data)
-
-    assert "y_missing" in model.named_vars
-
-    test_point = model.initial_point()
-    assert not np.isnan(model.compile_logp()(test_point))
-
-    with model:
-        prior_trace = pm.sample_prior_predictive(return_inferencedata=False)
-    assert {"x", "y"} <= set(prior_trace.keys())
-
-
-def test_missing_dual_observations():
-    with pm.Model() as model:
-        obs1 = np.ma.masked_values([1, 2, -1, 4, -1], value=-1)
-        obs2 = np.ma.masked_values([-1, -1, 6, -1, 8], value=-1)
-        beta1 = pm.Normal("beta1", 1, 1)
-        beta2 = pm.Normal("beta2", 2, 1)
-        latent = pm.Normal("theta", size=5)
-        with pytest.warns(ImputationWarning):
-            ovar1 = pm.Normal("o1", mu=beta1 * latent, observed=obs1)
-        with pytest.warns(ImputationWarning):
-            ovar2 = pm.Normal("o2", mu=beta2 * latent, observed=obs2)
-
-        prior_trace = pm.sample_prior_predictive(return_inferencedata=False)
-        assert {"beta1", "beta2", "theta", "o1", "o2"} <= set(prior_trace.keys())
-        # TODO: Assert something
-        with warnings.catch_warnings():
-            warnings.filterwarnings("ignore", ".*number of samples.*", UserWarning)
-            trace = pm.sample(chains=1, draws=50)
-
-
-def test_interval_missing_observations():
-    with pm.Model() as model:
-        obs1 = np.ma.masked_values([1, 2, -1, 4, -1], value=-1)
-        obs2 = np.ma.masked_values([-1, -1, 6, -1, 8], value=-1)
-
-        rng = aesara.shared(np.random.RandomState(2323), borrow=True)
-
-        with pytest.warns(ImputationWarning):
-            theta1 = pm.Uniform("theta1", 0, 5, observed=obs1, rng=rng)
-        with pytest.warns(ImputationWarning):
-            theta2 = pm.Normal("theta2", mu=theta1, observed=obs2, rng=rng)
-
-        assert "theta1_observed" in model.named_vars
-        assert "theta1_missing_interval__" in model.named_vars
-        assert not hasattr(
-            model.rvs_to_values[model.named_vars["theta1_observed"]].tag, "transform"
-        )
-
-        prior_trace = pm.sample_prior_predictive(return_inferencedata=False)
-
-        # Make sure the observed + missing combined deterministics have the
-        # same shape as the original observations vectors
-        assert prior_trace["theta1"].shape[-1] == obs1.shape[0]
-        assert prior_trace["theta2"].shape[-1] == obs2.shape[0]
-
-        # Make sure that the observed values are newly generated samples
-        assert np.all(np.var(prior_trace["theta1_observed"], 0) > 0.0)
-        assert np.all(np.var(prior_trace["theta2_observed"], 0) > 0.0)
-
-        # Make sure the missing parts of the combined deterministic matches the
-        # sampled missing and observed variable values
-        assert np.mean(prior_trace["theta1"][:, obs1.mask] - prior_trace["theta1_missing"]) == 0.0
-        assert np.mean(prior_trace["theta1"][:, ~obs1.mask] - prior_trace["theta1_observed"]) == 0.0
-        assert np.mean(prior_trace["theta2"][:, obs2.mask] - prior_trace["theta2_missing"]) == 0.0
-        assert np.mean(prior_trace["theta2"][:, ~obs2.mask] - prior_trace["theta2_observed"]) == 0.0
-
-        assert {"theta1", "theta2"} <= set(prior_trace.keys())
-
-        trace = pm.sample(
-            chains=1, draws=50, compute_convergence_checks=False, return_inferencedata=False
-        )
-
-        assert np.all(0 < trace["theta1_missing"].mean(0))
-        assert np.all(0 < trace["theta2_missing"].mean(0))
-        assert "theta1" not in trace.varnames
-        assert "theta2" not in trace.varnames
-
-        # Make sure that the observed values are newly generated samples and that
-        # the observed and deterministic matche
-        pp_idata = pm.sample_posterior_predictive(trace)
-        pp_trace = pp_idata.posterior_predictive.stack(sample=["chain", "draw"]).transpose(
-            "sample", ...
-        )
-        assert np.all(np.var(pp_trace["theta1"], 0) > 0.0)
-        assert np.all(np.var(pp_trace["theta2"], 0) > 0.0)
-        assert np.isclose(
-            np.mean(pp_trace["theta1"][:, ~obs1.mask] - pp_trace["theta1_observed"]), 0
-        )
-        assert np.isclose(
-            np.mean(pp_trace["theta2"][:, ~obs2.mask] - pp_trace["theta2_observed"]), 0
-        )
-
-
-def test_double_counting():
-    with pm.Model(check_bounds=False) as m1:
-        x = pm.Gamma("x", 1, 1, size=4)
-
-    logp_val = m1.compile_logp()({"x_log__": np.array([0, 0, 0, 0])})
-    assert logp_val == -4.0
-
-    with pm.Model(check_bounds=False) as m2:
-        with pytest.warns(ImputationWarning):
-            x = pm.Gamma("x", 1, 1, observed=[1, 1, 1, np.nan])
-
-    logp_val = m2.compile_logp()({"x_missing_log__": np.array([0])})
-    assert logp_val == -4.0
-
-
-def test_missing_logp():
-    with pm.Model() as m:
-        theta1 = pm.Normal("theta1", 0, 5, observed=[0, 1, 2, 3, 4])
-        theta2 = pm.Normal("theta2", mu=theta1, observed=[0, 1, 2, 3, 4])
-    m_logp = m.compile_logp()({})
-
-    with pm.Model() as m_missing:
-        with pytest.warns(ImputationWarning):
-            theta1 = pm.Normal("theta1", 0, 5, observed=np.array([0, 1, np.nan, 3, np.nan]))
-            theta2 = pm.Normal(
-                "theta2", mu=theta1, observed=np.array([np.nan, np.nan, 2, np.nan, 4])
-            )
-    m_missing_logp = m_missing.compile_logp()(
-        {"theta1_missing": [2, 4], "theta2_missing": [0, 1, 3]}
-    )
-
-    assert m_logp == m_missing_logp
-
-
-def test_missing_multivariate():
-    """Test model with missing variables whose transform changes base shape still works"""
-
-    with pm.Model() as m_miss:
-        with pytest.raises(
-            NotImplementedError,
-            match="Automatic inputation is only supported for univariate RandomVariables",
-        ):
+    def test_missing_basic(self, missing_data):
+        with pm.Model() as model:
+            x = pm.Normal("x", 1, 1)
             with pytest.warns(ImputationWarning):
-                x = pm.Dirichlet(
-                    "x", a=[1, 2, 3], observed=np.array([[0.3, 0.3, 0.4], [np.nan, np.nan, np.nan]])
-                )
+                _ = pm.Normal("y", x, 1, observed=missing_data)
 
-    # TODO: Test can be used when local_subtensor_rv_lift supports multivariate distributions
-    # from pymc.distributions.transforms import simplex
-    #
-    # with pm.Model() as m_unobs:
-    #     x = pm.Dirichlet("x", a=[1, 2, 3])
-    #
-    # inp_vals = simplex.forward(np.array([0.3, 0.3, 0.4])).eval()
-    # assert np.isclose(
-    #     m_miss.compile_logp()({"x_missing_simplex__": inp_vals}),
-    #     m_unobs.compile_logp(jacobian=False)({"x_simplex__": inp_vals}) * 2,
-    # )
+        assert "y_missing" in model.named_vars
 
+        test_point = model.initial_point()
+        assert not np.isnan(model.compile_logp()(test_point))
 
-def test_missing_vector_parameter():
-    with pm.Model() as m:
-        with pytest.warns(ImputationWarning):
-            x = pm.Normal(
-                "x",
-                np.array([-10, 10]),
-                0.1,
-                observed=np.array([[np.nan, 10], [-10, np.nan], [np.nan, np.nan]]),
+        with model:
+            ipr = pm.sample_prior_predictive()
+        assert {"x", "y"} <= set(ipr.prior.keys())
+
+    def test_missing_with_predictors(self):
+        predictors = np.array([0.5, 1, 0.5, 2, 0.3])
+        data = np.ma.masked_values([1, 2, -1, 4, -1], value=-1)
+        with pm.Model() as model:
+            x = pm.Normal("x", 1, 1)
+            with pytest.warns(ImputationWarning):
+                y = pm.Normal("y", x * predictors, 1, observed=data)
+
+        assert "y_missing" in model.named_vars
+
+        test_point = model.initial_point()
+        assert not np.isnan(model.compile_logp()(test_point))
+
+        with model:
+            ipr = pm.sample_prior_predictive()
+        assert {"x", "y"} <= set(ipr.prior.keys())
+
+    def test_missing_dual_observations(self):
+        with pm.Model() as model:
+            obs1 = np.ma.masked_values([1, 2, -1, 4, -1], value=-1)
+            obs2 = np.ma.masked_values([-1, -1, 6, -1, 8], value=-1)
+            beta1 = pm.Normal("beta1", 1, 1)
+            beta2 = pm.Normal("beta2", 2, 1)
+            latent = pm.Normal("theta", size=5)
+            with pytest.warns(ImputationWarning):
+                ovar1 = pm.Normal("o1", mu=beta1 * latent, observed=obs1)
+            with pytest.warns(ImputationWarning):
+                ovar2 = pm.Normal("o2", mu=beta2 * latent, observed=obs2)
+
+            prior_trace = pm.sample_prior_predictive(return_inferencedata=False)
+            assert {"beta1", "beta2", "theta", "o1", "o2"} <= set(prior_trace.keys())
+            # TODO: Assert something
+            with warnings.catch_warnings():
+                warnings.filterwarnings("ignore", ".*number of samples.*", UserWarning)
+                trace = pm.sample(chains=1, tune=5, draws=50)
+
+    def test_interval_missing_observations(self):
+        rng = np.random.default_rng(1198)
+
+        with pm.Model() as model:
+            obs1 = np.ma.masked_values([1, 2, -1, 4, -1], value=-1)
+            obs2 = np.ma.masked_values([-1, -1, 6, -1, 8], value=-1)
+
+            with pytest.warns(ImputationWarning):
+                theta1 = pm.Uniform("theta1", 0, 5, observed=obs1)
+            with pytest.warns(ImputationWarning):
+                theta2 = pm.Normal("theta2", mu=theta1, observed=obs2)
+
+            assert isinstance(model.rvs_to_transforms[model["theta1_missing"]], IntervalTransform)
+            assert model.rvs_to_transforms[model["theta1_observed"]] is None
+
+            prior_trace = pm.sample_prior_predictive(random_seed=rng, return_inferencedata=False)
+            assert set(prior_trace.keys()) == {
+                "theta1",
+                "theta1_observed",
+                "theta1_missing",
+                "theta2",
+                "theta2_observed",
+                "theta2_missing",
+            }
+
+            # Make sure the observed + missing combined deterministics have the
+            # same shape as the original observations vectors
+            assert prior_trace["theta1"].shape[-1] == obs1.shape[0]
+            assert prior_trace["theta2"].shape[-1] == obs2.shape[0]
+
+            # Make sure that the observed values are newly generated samples
+            assert np.all(np.var(prior_trace["theta1_observed"], 0) > 0.0)
+            assert np.all(np.var(prior_trace["theta2_observed"], 0) > 0.0)
+
+            # Make sure the missing parts of the combined deterministic matches the
+            # sampled missing and observed variable values
+            assert (
+                np.mean(prior_trace["theta1"][:, obs1.mask] - prior_trace["theta1_missing"]) == 0.0
+            )
+            assert (
+                np.mean(prior_trace["theta1"][:, ~obs1.mask] - prior_trace["theta1_observed"])
+                == 0.0
+            )
+            assert (
+                np.mean(prior_trace["theta2"][:, obs2.mask] - prior_trace["theta2_missing"]) == 0.0
+            )
+            assert (
+                np.mean(prior_trace["theta2"][:, ~obs2.mask] - prior_trace["theta2_observed"])
+                == 0.0
             )
-    x_draws = x.eval()
-    assert x_draws.shape == (3, 2)
-    assert np.all(x_draws[:, 0] < 0)
-    assert np.all(x_draws[:, 1] > 0)
-    assert np.isclose(
-        m.compile_logp()({"x_missing": np.array([-10, 10, -10, 10])}),
-        st.norm(scale=0.1).logpdf(0) * 6,
-    )
 
+            trace = pm.sample(
+                chains=1,
+                draws=50,
+                compute_convergence_checks=False,
+                return_inferencedata=False,
+                random_seed=rng,
+            )
+            assert set(trace.varnames) == {
+                "theta1",
+                "theta1_missing",
+                "theta1_missing_interval__",
+                "theta2",
+                "theta2_missing",
+            }
 
-def test_missing_symmetric():
-    """Check that logp works when partially observed variable have equal observed and
-    unobserved dimensions.
+            # Make sure that the missing values are newly generated samples and that
+            # the observed and deterministic match
+            assert np.all(0 < trace["theta1_missing"].mean(0))
+            assert np.all(0 < trace["theta2_missing"].mean(0))
+            assert np.isclose(np.mean(trace["theta1"][:, obs1.mask] - trace["theta1_missing"]), 0)
+            assert np.isclose(np.mean(trace["theta2"][:, obs2.mask] - trace["theta2_missing"]), 0)
 
-    This would fail in a previous implementation because the two variables would be
-    equivalent and one of them would be discarded during MergeOptimization while
-    buling the logp graph
-    """
-    with pm.Model() as m:
-        with pytest.warns(ImputationWarning):
-            x = pm.Gamma("x", alpha=3, beta=10, observed=np.array([1, np.nan]))
+            # Make sure that the observed values are unchanged
+            assert np.allclose(np.var(trace["theta1"][:, ~obs1.mask], 0), 0.0)
+            assert np.allclose(np.var(trace["theta2"][:, ~obs2.mask], 0), 0.0)
+            np.testing.assert_array_equal(trace["theta1"][0][~obs1.mask], obs1[~obs1.mask])
+            np.testing.assert_array_equal(trace["theta2"][0][~obs2.mask], obs1[~obs2.mask])
 
-    x_obs_rv = m["x_observed"]
-    x_obs_vv = m.rvs_to_values[x_obs_rv]
+            pp_idata = pm.sample_posterior_predictive(trace, random_seed=rng)
+            pp_trace = pp_idata.posterior_predictive.stack(sample=["chain", "draw"]).transpose(
+                "sample", ...
+            )
+            assert set(pp_trace.keys()) == {
+                "theta1",
+                "theta1_observed",
+                "theta2",
+                "theta2_observed",
+            }
 
-    x_unobs_rv = m["x_missing"]
-    x_unobs_vv = m.rvs_to_values[x_unobs_rv]
+            # Make sure that the observed values are newly generated samples and that
+            # the observed and deterministic match
+            assert np.all(np.var(pp_trace["theta1"], 0) > 0.0)
+            assert np.all(np.var(pp_trace["theta2"], 0) > 0.0)
+            assert np.isclose(
+                np.mean(pp_trace["theta1"][:, ~obs1.mask] - pp_trace["theta1_observed"]), 0
+            )
+            assert np.isclose(
+                np.mean(pp_trace["theta2"][:, ~obs2.mask] - pp_trace["theta2_observed"]), 0
+            )
 
-    logp = pm.joint_logp([x_obs_rv, x_unobs_rv], {x_obs_rv: x_obs_vv, x_unobs_rv: x_unobs_vv})
-    logp_inputs = list(graph_inputs([logp]))
-    assert x_obs_vv in logp_inputs
-    assert x_unobs_vv in logp_inputs
+    def test_missing_logp1(self):
+        with pm.Model(check_bounds=False) as m1:
+            x = pm.Gamma("x", 1, 1, size=4)
+
+        logp_val = m1.compile_logp()({"x_log__": np.array([0, 0, 0, 0])})
+        assert logp_val == -4.0
+
+        with pm.Model(check_bounds=False) as m2:
+            with pytest.warns(ImputationWarning):
+                x = pm.Gamma("x", 1, 1, observed=[1, 1, 1, np.nan])
+
+        logp_val = m2.compile_logp()({"x_missing_log__": np.array([0])})
+        assert logp_val == -4.0
+
+    def test_missing_logp2(self):
+        with pm.Model() as m:
+            theta1 = pm.Normal("theta1", 0, 5, observed=[0, 1, 2, 3, 4])
+            theta2 = pm.Normal("theta2", mu=theta1, observed=[0, 1, 2, 3, 4])
+        m_logp = m.compile_logp()({})
+
+        with pm.Model() as m_missing:
+            with pytest.warns(ImputationWarning):
+                theta1 = pm.Normal("theta1", 0, 5, observed=np.array([0, 1, np.nan, 3, np.nan]))
+                theta2 = pm.Normal(
+                    "theta2", mu=theta1, observed=np.array([np.nan, np.nan, 2, np.nan, 4])
+                )
+        m_missing_logp = m_missing.compile_logp()(
+            {"theta1_missing": [2, 4], "theta2_missing": [0, 1, 3]}
+        )
+
+        assert m_logp == m_missing_logp
+
+    def test_missing_multivariate(self):
+        """Test model with missing variables whose transform changes base shape still works"""
+
+        with pm.Model() as m_miss:
+            with pytest.raises(
+                NotImplementedError,
+                match="Automatic inputation is only supported for univariate RandomVariables",
+            ):
+                with pytest.warns(ImputationWarning):
+                    x = pm.Dirichlet(
+                        "x",
+                        a=[1, 2, 3],
+                        observed=np.array([[0.3, 0.3, 0.4], [np.nan, np.nan, np.nan]]),
+                    )
+
+        # TODO: Test can be used when local_subtensor_rv_lift supports multivariate distributions
+        # from pymc.distributions.transforms import simplex
+        #
+        # with pm.Model() as m_unobs:
+        #     x = pm.Dirichlet("x", a=[1, 2, 3])
+        #
+        # inp_vals = simplex.forward(np.array([0.3, 0.3, 0.4])).eval()
+        # assert np.isclose(
+        #     m_miss.compile_logp()({"x_missing_simplex__": inp_vals}),
+        #     m_unobs.compile_logp(jacobian=False)({"x_simplex__": inp_vals}) * 2,
+        # )
+
+    def test_missing_vector_parameter(self):
+        with pm.Model() as m:
+            with pytest.warns(ImputationWarning):
+                x = pm.Normal(
+                    "x",
+                    np.array([-10, 10]),
+                    0.1,
+                    observed=np.array([[np.nan, 10], [-10, np.nan], [np.nan, np.nan]]),
+                )
+        x_draws = x.eval()
+        assert x_draws.shape == (3, 2)
+        assert np.all(x_draws[:, 0] < 0)
+        assert np.all(x_draws[:, 1] > 0)
+        assert np.isclose(
+            m.compile_logp()({"x_missing": np.array([-10, 10, -10, 10])}),
+            st.norm(scale=0.1).logpdf(0) * 6,
+        )
+
+    def test_missing_symmetric(self):
+        """Check that logp works when partially observed variable have equal observed and
+        unobserved dimensions.
+
+        This would fail in a previous implementation because the two variables would be
+        equivalent and one of them would be discarded during MergeOptimization while
+        building the logp graph
+        """
+        with pm.Model() as m:
+            with pytest.warns(ImputationWarning):
+                x = pm.Gamma("x", alpha=3, beta=10, observed=np.array([1, np.nan]))
+
+        x_obs_rv = m["x_observed"]
+        x_obs_vv = m.rvs_to_values[x_obs_rv]
+
+        x_unobs_rv = m["x_missing"]
+        x_unobs_vv = m.rvs_to_values[x_unobs_rv]
+
+        logp = _joint_logp(
+            [x_obs_rv, x_unobs_rv],
+            rvs_to_values={x_obs_rv: x_obs_vv, x_unobs_rv: x_unobs_vv},
+            rvs_to_transforms={},
+            rvs_to_total_sizes={},
+        )
+        logp_inputs = list(graph_inputs(logp))
+        assert x_obs_vv in logp_inputs
+        assert x_unobs_vv in logp_inputs
+
+    def test_dims(self):
+        """Test that we don't propagate dims to the subcomponents of a partially
+        observed RV
+
+        See https://github.com/pymc-devs/pymc/issues/6177
+        """
+        data = np.array([np.nan] * 3 + [0] * 7)
+        with pm.Model(coords={"observed": range(10)}) as model:
+            with pytest.warns(ImputationWarning):
+                x = pm.Normal("x", observed=data, dims=("observed",))
+        assert model.named_vars_to_dims == {"x": ("observed",)}
+
+    def test_error_non_random_variable(self):
+        data = np.array([np.nan] * 3 + [0] * 7)
+        with pm.Model() as model:
+            msg = "x of type <class 'pymc.distributions.censored.CensoredRV'> is not supported"
+            with pytest.raises(NotImplementedError, match=msg):
+                x = pm.Censored(
+                    "x",
+                    pm.Normal.dist(),
+                    lower=0,
+                    upper=10,
+                    observed=data,
+                )
 
 
 class TestShared(SeededTest):
@@ -1369,3 +1435,59 @@ class TestShared(SeededTest):
             assert np.all(
                 np.isclose(model.compile_logp(sum=False)({}), st.norm().logpdf(data_values))
             )
+
+
+def test_tag_future_warning_model():
+    # Test no unexpected warnings
+    with warnings.catch_warnings():
+        warnings.simplefilter("error")
+
+        model = pm.Model()
+
+        x = at.random.normal()
+        x.tag.something_else = "5"
+        x.tag.test_value = 0
+        assert not isinstance(x.tag, _FutureWarningValidatingScratchpad)
+
+        # Test that model changes the tag type, but copies exsiting contents
+        x = model.register_rv(x, name="x", transform=log)
+        assert isinstance(x.tag, _FutureWarningValidatingScratchpad)
+        assert x.tag.something_else == "5"
+        assert x.tag.test_value == 0
+
+        # Test expected warnings
+        with pytest.warns(FutureWarning, match="model.rvs_to_values"):
+            x_value = x.tag.value_var
+
+        assert isinstance(x_value.tag, _FutureWarningValidatingScratchpad)
+        with pytest.warns(FutureWarning, match="model.rvs_to_transforms"):
+            transform = x_value.tag.transform
+        assert transform is log
+
+        with pytest.raises(AttributeError):
+            x.tag.observations
+
+        with pytest.warns(FutureWarning, match="model.rvs_to_total_sizes"):
+            total_size = x.tag.total_size
+        assert total_size is None
+
+        # Cloning a node will keep the same tag type and contents
+        y = x.owner.clone().default_output()
+        assert y is not x
+        assert y.tag is not x.tag
+        assert isinstance(y.tag, _FutureWarningValidatingScratchpad)
+        y = model.register_rv(y, name="y", observed=5)
+        assert isinstance(y.tag, _FutureWarningValidatingScratchpad)
+
+        # Test expected warnings
+        with pytest.warns(FutureWarning, match="model.rvs_to_values"):
+            y_value = y.tag.value_var
+        with pytest.warns(FutureWarning, match="model.rvs_to_values"):
+            y_obs = y.tag.observations
+        assert y_value is y_obs
+        assert y_value.eval() == 5
+
+        assert isinstance(y_value.tag, _FutureWarningValidatingScratchpad)
+        with pytest.warns(FutureWarning, match="model.rvs_to_total_sizes"):
+            total_size = y.tag.total_size
+        assert total_size is None
