@@ -49,16 +49,22 @@ pymc/math.py
 pymc/ode/__init__.py
 pymc/ode/ode.py
 pymc/ode/utils.py
-pymc/parallel_sampling.py
 pymc/plots/__init__.py
-pymc/sampling.py
+pymc/sampling_jax.py
+pymc/sampling/__init__.py
+pymc/sampling/forward.py
+pymc/sampling/mcmc.py
+pymc/sampling/parallel.py
+pymc/sampling/population.py
 pymc/smc/__init__.py
 pymc/smc/sampling.py
 pymc/smc/kernels.py
 pymc/stats/__init__.py
 pymc/stats/convergence.py
 pymc/step_methods/__init__.py
+pymc/step_methods/arraystep.py
 pymc/step_methods/compound.py
+pymc/step_methods/metropolis.py
 pymc/step_methods/hmc/__init__.py
 pymc/step_methods/hmc/base_hmc.py
 pymc/step_methods/hmc/hmc.py
@@ -140,7 +146,7 @@ def check_no_unexpected_results(mypy_lines: Iterator[str]):
     all_files = {
         str(fp).replace(str(DP_ROOT), "").strip(os.sep).replace(os.sep, "/")
         for fp in DP_ROOT.glob("pymc/**/*.py")
-        if not "tests" in str(fp)
+        if "tests" not in str(fp)
     }
     failing = set(df.reset_index().file.str.replace(os.sep, "/", regex=False))
     if not failing.issubset(all_files):
@@ -166,10 +172,10 @@ def check_no_unexpected_results(mypy_lines: Iterator[str]):
         print("You can run `python scripts/run_mypy.py --verbose` to reproduce this test locally.")
         sys.exit(1)
 
-    if unexpected_passing == {"pymc/sampling_jax.py"}:
-        print("Letting you know that 'pymc/sampling_jax.py' unexpectedly passed.")
+    if unexpected_passing == {"pymc/sampling/jax.py"}:
+        print("Letting you know that 'pymc/sampling/jax.py' unexpectedly passed.")
         print("But this file is known to sometimes pass and sometimes not.")
-        print("Unless you tried to resolve problems in sampling_jax.py just ignore this message.")
+        print("Unless you tried to resolve problems in sampling/jax.py just ignore this message.")
     elif unexpected_passing:
         print("!!!!!!!!!")
         print(f"{len(unexpected_passing)} files unexpectedly passed the type checks:")
